@@ -23,6 +23,10 @@
 
 static const char* FILE_NAME = "Sys/IO.c";
 
+#if defined(_WIN32)
+  #include "Sys-Windows.h"
+#endif
+
 int32_t SPVM__Sys__IO__fopen(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
@@ -41,8 +45,22 @@ int32_t SPVM__Sys__IO__fopen(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* mode = env->get_chars(env, stack, obj_mode);
   
-  FILE* stream = fopen(path, mode);
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
   
+  wchar_t* mode_w = utf8_to_win_wchar(env, stack, mode, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  FILE* stream = _wfopen(path_w, mode_w);
+#else
+  FILE* stream = fopen(path, mode);
+#endif
+
   if (!stream) {
     env->die(env, stack, "[System Error]fopen() failed:%s. $path is \"%s\". $mode is \"%s\"", env->strerror_nolen(env, stack, errno), path, mode, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -524,6 +542,8 @@ int32_t SPVM__Sys__IO__setvbuf(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__IO__open(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   int32_t flags = stack[1].ival;
@@ -536,7 +556,17 @@ int32_t SPVM__Sys__IO__open(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t fd = _wopen(path_w, flags, mode);
+#else
   int32_t fd = open(path, flags, mode);
+#endif
+
   if (fd == -1) {
     env->die(env, stack, "[System Error]open() failed:%s. $path is \"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -763,7 +793,9 @@ int32_t SPVM__Sys__IO__flock(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__IO__access(SPVM_ENV* env, SPVM_VALUE* stack) {
-
+  
+  int32_t error_id = 0;
+  
   void* obj_pathname = stack[0].oval;
   
   int32_t mode = stack[1].ival;
@@ -774,7 +806,16 @@ int32_t SPVM__Sys__IO__access(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* pathname = env->get_chars(env, stack, obj_pathname);
   
+#if defined(_WIN32)
+  wchar_t* pathname_w = utf8_to_win_wchar(env, stack, pathname, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = _waccess(pathname_w, mode);
+#else
   int32_t status = access(pathname, mode);
+#endif
   
   if (status == -1) {
     const char* pathname = env->get_chars(env, stack, obj_pathname);
@@ -821,6 +862,8 @@ int32_t SPVM__Sys__IO__faccessat(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__IO__truncate(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   int64_t length = stack[1].lval;
@@ -834,7 +877,21 @@ int32_t SPVM__Sys__IO__truncate(SPVM_ENV* env, SPVM_VALUE* stack) {
     return env->die(env, stack, "The length $length must be less than or equal to 0.", __func__, FILE_NAME, __LINE__);
   }
   
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t fd = _wopen(path_w, O_WRONLY);
+  int32_t status = ftruncate(fd, length);
+  if (!(fd == -1)) {
+    close(fd);
+  }
+#else
   int32_t status = truncate(path, length);
+#endif
+
   if (status == -1) {
     env->die(env, stack, "[System Error]truncate() failed:%s. $path is \"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -847,6 +904,8 @@ int32_t SPVM__Sys__IO__truncate(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   int32_t mode = stack[1].ival;
@@ -858,7 +917,12 @@ int32_t SPVM__Sys__IO__mkdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   const char* path = env->get_chars(env, stack, obj_path);
   
 #if defined(_WIN32)
-  int32_t status = _mkdir(path);
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = _wmkdir(path_w);
 #else
   int32_t status = mkdir(path, mode);
 #endif
@@ -886,6 +950,8 @@ int32_t SPVM__Sys__IO__umask(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__IO__rmdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   if (!obj_path) {
@@ -894,7 +960,17 @@ int32_t SPVM__Sys__IO__rmdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = _wrmdir(path_w);
+#else
   int32_t status = rmdir(path);
+#endif
+
   if (status == -1) {
     env->die(env, stack, "[System Error]rmdir() failed:%s. $path is \"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -965,34 +1041,56 @@ int32_t SPVM__Sys__IO__getcwd(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t size = stack[1].ival;
   
+  if (obj_buf) {
+    return env->die(env, stack, "The buffer $buf must be undef.", __func__, FILE_NAME, __LINE__);
+  }
+  
   if (!(size >= 0)) {
     return env->die(env, stack, "The size $size must be greater than or equal to 0.", __func__, FILE_NAME, __LINE__);
   }
   
-  char* ret_buf;
-  if (obj_buf) {
-    char* buf = (char*)env->get_chars(env, stack, obj_buf);
-    int32_t buf_length = env->length(env, stack, obj_buf);
-    if (!(size <= buf_length)) {
-      return env->die(env, stack, "The size $size must be less than or equal to the lenght of $buf.", __func__, FILE_NAME, __LINE__);
-    }
-    
-    ret_buf = getcwd(buf, size);
-  }
-  else {
-    ret_buf = getcwd(NULL, size);
-    if (ret_buf) {
-      obj_buf = env->new_string(env, stack, ret_buf, strlen(ret_buf));
-      free(ret_buf);
-    }
-  }
+  void* free_object = NULL;
   
-  if (!ret_buf) {
+#if defined(_WIN32)
+  
+  wchar_t* ret_w = _wgetcwd(NULL, size);
+  free_object = ret_w;
+  
+  char* ret = (char*)win_wchar_to_utf8(env, stack, ret_w, &error_id, __func__, FILE_NAME, __LINE__);
+  
+  if (error_id) {
+    goto END_OF_FUNC;
+  }
+#else
+  char* ret = getcwd(NULL, size);
+  free_object = ret;
+#endif
+
+  if (!ret) {
     env->die(env, stack, "[System Error]getcwd() failed:%s.", env->strerror_nolen(env, stack, errno), __func__, FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
+    error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
+    goto END_OF_FUNC;
   }
   
-  stack[0].oval = obj_buf;
+  END_OF_FUNC:
+  
+  ;
+  
+  void* obj_ret = NULL;
+  
+  if (ret) {
+    obj_ret = env->new_string(env, stack, ret, strlen(ret));
+  }
+  
+  if (free_object) {
+    free(free_object);
+  }
+  
+  if (error_id) {
+    return error_id;
+  }
+  
+  stack[0].oval = obj_ret;
   
   return 0;
 }
@@ -1008,36 +1106,44 @@ int32_t SPVM__Sys__IO___getdcwd(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_buffer = stack[1].oval;
   
+  if (obj_buffer) {
+    return env->die(env, stack, "The buffer $buffer must be undef.", __func__, FILE_NAME, __LINE__);
+  }
+  
   int32_t maxlen = stack[2].ival;
   
   if (!(maxlen > 0)) {
     return env->die(env, stack, "The max length $maxlen must be greater than 0.", __func__, FILE_NAME, __LINE__);
   }
   
-  char* ret_buffer;
-  if (obj_buffer) {
-    char* buffer = (char*)env->get_chars(env, stack, obj_buffer);
-    int32_t buffer_length = env->length(env, stack, obj_buffer);
-    if (!(maxlen <= buffer_length)) {
-      return env->die(env, stack, "The max length $maxlen must be less than or equal to the lenght of the buffer $buffer.", __func__, FILE_NAME, __LINE__);
-    }
-    
-    ret_buffer = _getdcwd(drive, buffer, maxlen);
-  }
-  else {
-    ret_buffer = _getdcwd(drive, NULL, maxlen);
-    if (ret_buffer) {
-      obj_buffer = env->new_string(env, stack, ret_buffer, strlen(ret_buffer));
-      free(ret_buffer);
-    }
+  wchar_t* ret_w = _wgetdcwd(drive, NULL, maxlen);
+  
+  void* free_object = ret_w;
+  
+  char* ret = (char*)win_wchar_to_utf8(env, stack, ret_w, &error_id, __func__, FILE_NAME, __LINE__);
+  
+  if (error_id) {
+    goto END_OF_FUNC;
   }
   
-  if (!ret_buffer) {
+  if (!ret) {
     env->die(env, stack, "[System Error]_getdcwd() failed:%s.", env->strerror_nolen(env, stack, errno), __func__, FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
+    error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
-  stack[0].oval = obj_buffer;
+  END_OF_FUNC:
+  
+  if (free_object) {
+    free(free_object);
+  }
+  
+  if (error_id) {
+    return error_id;
+  }
+  
+  void* obj_ret = env->new_string(env, stack, ret, strlen(ret));
+  
+  stack[0].oval = obj_ret;
   
   return 0;
 #endif
@@ -1083,56 +1189,27 @@ int32_t SPVM__Sys__IO__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
 #endif
 }
 
-int32_t SPVM__Sys__IO___fullpath(SPVM_ENV* env, SPVM_VALUE* stack) {
-#if !defined(_WIN32)
-  env->die(env, stack, "Sys::IO#_fullpath method is not supported in this system(!defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
-  return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
-#else
-  void* obj_absPath = stack[0].oval;
-  
-  void* obj_relPath = stack[1].oval;
-
-  if (!obj_relPath) {
-    return env->die(env, stack, "The relative path $relPath must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  char* relPath = (char*)env->get_chars(env, stack, obj_relPath);
-  
-  int32_t maxLength = stack[2].ival;
-  
-  char* ret_absPath;
-  if (obj_absPath) {
-    char* absPath = (char*)env->get_chars(env, stack, obj_absPath);
-    ret_absPath = _fullpath(absPath, relPath, maxLength);
-  }
-  else {
-    ret_absPath = _fullpath(NULL, relPath, 0);
-    if (ret_absPath) {
-      obj_absPath = env->new_string(env, stack, ret_absPath, strlen(ret_absPath));
-      free(ret_absPath);
-    }
-  }
-  
-  if (!ret_absPath) {
-    env->die(env, stack, "[System Error]_fullpath() failed:%s. $relPath is \"%s\".", env->strerror_nolen(env, stack, errno), relPath, __func__, FILE_NAME, __LINE__);
-    return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
-  }
-  
-  stack[0].oval = obj_absPath;
-
-  return 0;
-#endif
-}
-
 int32_t SPVM__Sys__IO__chdir(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   if (!obj_path) {
     return env->die(env, stack, "The path $path must be defined.", __func__, FILE_NAME, __LINE__);
   }
   const char* path = env->get_chars(env, stack, obj_path);
   
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = _wchdir(path_w);
+#else
   int32_t status = chdir(path);
+#endif
+
   if (status == -1) {
     env->die(env, stack, "[System Error]chdir() failed:%s. $path is \"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -1145,6 +1222,8 @@ int32_t SPVM__Sys__IO__chdir(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__IO__chmod(SPVM_ENV* env, SPVM_VALUE* stack) {
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   int32_t mode = stack[1].ival;
@@ -1154,7 +1233,17 @@ int32_t SPVM__Sys__IO__chmod(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* path = env->get_chars(env, stack, obj_path);
   
+#if defined(_WIN32)
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = _wchmod(path_w, mode);
+#else
   int32_t status = chmod(path, mode);
+#endif
+
   if (status == -1) {
     env->die(env, stack, "[System Error]chmod() failed:%s. $path is \"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
